#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

##
# PuppetX is the toplevel namespace for working with Arista EOS nodes
module PuppetX
  ##
  # Eos is module namesapce for working with the EOS command API
  module Eos
    ##
    # The System class provides management of system level functions
    #
    class System
      ##
      # Initializes a new instance of System.
      #
      # @param [PuppetX::Eos::Eapi] api An instance of Eapi
      #
      # @return [PuppetX::Eos::System] instance
      def initialize(api)
        @api = api
      end

      ##
      # Returns the configured system hostname from the running-config
      #
      # @return [String]  the configured system hostname
      def get_hostname
        result = @api.enable('show hostname')
        result.first['hostname'].gsub("\n", '')
      end

      ##
      # Returns the configured domain-name from the running-config
      #
      # @return [String] the configured domain-name
      def get_domain_name
        result = @api.enable('show ip domain-name', 'text')
        result.first['output'].gsub("\n", '')
      end

      ##
      # Returns the configure domain-list from the running-config.  If no
      # domain-list has been created, then an empty array is returned
      #
      # @return [Array] the list of configured domain names
      def get_domain_list
        result = @api.enable('show running-config section ip\sdomain-list',
                             'text')
        result.first['output'].scan(/(?<=list\s).*$/)
      end

      ##
      # Returns the list of configured name-servers from the running-config
      #
      # @return [Array] the list of configured name servers
      def get_name_servers
        result = @api.enable('show ip name-server', 'text')
        result.first['output'].split("\n")
      end

      ##
      # Configures the system hostname
      #
      # @param [String] name The name to configure the hostname to
      #
      # @return [Boolean] True if the commands succeed otherwise False
      def set_hostname(name)
        @api.config("hostname #{name}") == [{}]
      end

      ##
      # Configures the system domain-name
      #
      # @param [String] name The name to configure the domain-name to
      #
      # @return [Boolean] True if the commands succeed otherwise False
      def set_domain_name(name)
        @api.config("ip domain-name #{name}")
      end

      ##
      # Configures the system name servers
      #
      # @param [Array] list of name-server values
      #
      # @return [Boolean] True if the commands succeed otherwise False
      def set_name_servers(servers)
        servers.each do |srv|
          resp = @api.config("ip name-server #{srv}")
          return false unless resp == [{}]
        end
      end

      ##
      # Configures the list of domain suffixes to search
      #
      # @param[Array] list of domain-name values
      #
      # @return [Boolean] True if the commands succeed otherwise False
      def set_domain_list(domains)
        domains.each do |name|
          resp = @api.config("ip domain-list #{name}")
          return false unless resp == [{}]
        end
      end
    end
  end
end
